/// <reference types="node" />
/// <reference types="node" />
import * as http from "http";
import { Verb } from "./tsrequest";
import { Nullable, StringDictionary, TSDictionary, uint16 } from "./types";
import { TSStaticWebSiteOptions } from "./tsservercomp";
import { TSDate } from "./tsdate";
import { TSColor } from "./tscolor";
/**
 * This is a minimal singleton HTTP server class provided for testing
 *
 * You start a new async API server on default port 3000 by a single line :
 *   TSServer.start({ ... my endpoints definitions dictionary ... }) ;
 *
 */
export interface TSServerRequest {
    method: Verb;
    url: URL;
    parameters: TSParameterDictionary;
    query: TSQueryDictionary;
    message: http.IncomingMessage;
}
export type TSServerResponse = http.ServerResponse;
export type TSEndPointManager = (req: TSServerRequest, resp: TSServerResponse) => Promise<void>;
export interface TSEndPoint {
    manager: TSEndPointManager;
    query?: TSQueryDefinition;
}
export type TSEndPoints = {
    [key in Verb]?: TSEndPoint | TSEndPointManager;
};
export type TSEndPointsDictionary = TSDictionary<TSEndPoints | TSEndPoint | TSEndPointManager>;
export type TSEndPointParameter = string | number | boolean | TSDate | TSColor;
export type TSParameterDictionary = TSDictionary<TSEndPointParameter>;
export type TSQueryValue = TSEndPointParameter;
export type TSQueryDictionary = TSDictionary<TSQueryValue>;
export declare const TSParametricTokenType: {
    readonly string: "string";
    readonly number: "number";
    readonly int: "int";
    readonly unsigned: "unsigned";
    readonly boolean: "boolean";
    readonly date: "date";
    readonly color: "color";
    readonly uuid: "uuid";
    readonly email: "email";
};
export type TSParametricTokenType = typeof TSParametricTokenType[keyof typeof TSParametricTokenType];
export declare enum TSServerStartStatus {
    AlreadyRunning = 0,
    HTTP = 1,
    HTTPS = 2
}
export interface TSParametricToken {
    name: string;
    type: TSParametricTokenType;
}
export interface TSQueryItem {
    type: TSParametricTokenType;
    mandatory?: boolean;
}
export type TSQueryDefinition = TSDictionary<TSParametricTokenType | TSQueryItem>;
/**
 * in order to create an https server, you only need to provide your valid
 * key and certificate in TSServerOptions
 */
export interface TSServerOptions extends TSStaticWebSiteOptions {
    host?: string;
    port?: uint16;
    rootPath?: string;
    webSites?: StringDictionary;
    logInfo?: boolean;
    key?: Nullable<Buffer>;
    certificate?: Nullable<Buffer>;
    maxHeaderSize?: Nullable<number>;
    tlsSecTimeout?: Nullable<number>;
    handshakeMsTimeout?: Nullable<number>;
    forceCloseMsTimeout?: Nullable<number>;
}
/**
 * other available options we don't considere for now in our small http/https server:
 *
 * > for http:
 *   ========
 *   - insecureHTTPParser?:Nullable<boolean>: Use an insecure HTTP parser that accepts invalid HTTP headers when true.
 *     Using the insecure parser should be avoided.
 *   - noDelay?:Nullable<boolean>: If set to `true`, it disables the use of
 *     Nagle's algorithm immediately after a new incoming connection is received
 *   - keepAlive?:Nullable<boolean>: If set to `true`, it enables keep-alive functionality
 *     on the socket immediately after a new incoming connection is received, similarly
 *     on what is done in `socket.setKeepAlive([enable][, initialDelay])`.
 *   - keepAliveInitialDelay?: Nullable<number>: If set to a positive number,
 *     it sets the initial delay before the first keepalive probe is sent on an idle socket.
 *
 * > for https:
 *   =========
 *
 *   - ca?:Nullable<sring|Buffer|Array<string|Buffer>>: override the trusted CA certificates
 *   – cert?:Nullable<sring|Buffer|Array<string|Buffer>>: Cert chains in PEM format.
 *   – sigalgs?: Nullable<string>: Colon-separated list of supported signature algorithms.
 *   – ciphers?: Nullable<string>: Cipher suite specification, replacing the default.
 *   - clientCertEngine?: Nullable<string>: Name of an OpenSSL engine which can provide
 *     the client certificate.
 *   - crl?:Nullable<sring|Buffer|Array<string|Buffer>>: PEM formatted CRLs
 *     (Certificate Revocation Lists).
 *   - dhparam?:Nullable<string|undefined>: Diffie Hellman parameters, required for Perfect Forward Secrecy.
 *   - ecdhCurve?:Nullable<string>: A string describing a named curve or a colon separated list of curve
 *     NIDs or names, for example P-521:P-384:P-256, to use for ECDH key agreement.
 *   - honorCipherOrder?:Nullable<boolean>: Attempt to use the server's cipher suite preferences
 *     instead of the client's. When true, causes SSL_OP_CIPHER_SERVER_PREFERENCE to be set in secureOptions.
 *   - keys?:Nullable<sring|Buffer|Array<string|Buffer>>: Private keys in PEM format.
 *   - privateKeyEngine?:Nullable<string>: Name of an OpenSSL engine to get private key from.
 *   - privateKeyIdentifier?:Nullable<string>: Identifier of a private key managed by an OpenSSL engine.
 *   - maxVersion?:Nullable<SecureVersion>: Optionally set the maximum TLS version to allow.
 *     One of `'TLSv1.3'`, `'TLSv1.2'`, `'TLSv1.1'`, or `'TLSv1'`
 *   - minVersion?:Nullable<SecureVersion>: IDEM for minimal version
 *   - passphrase?:Nullable<string>: Shared passphrase used for a single private key and/or a PFX
 *   - pfx?:Nullable<sring|Buffer|Array<string|Buffer|PxfObject>>: PFX or PKCS12 encoded private key
 *     and certificate chain. Alternative to providing key and cert individually.
 *   - secureOptions?:Nullable<number>: Value is a numeric bitmask of the `SSL_OP_*` options.
 *   - secureProtocol?:Nullable<string>: Legacy mechanism to select the TLS protocol version to use,
 *     it does support independent control of the minimum and maximum version. The possible values
 *     are listed as SSL_METHODS. Use the function names as strings.
 *   - sessionIdContext?:Nullable<string>: Opaque identifier used by servers to ensure session
 *     state is not shared between applications.
 *   - ticketKeys?:Nullable<Buffer>: 48-bytes of cryptographically strong pseudo-random data.
 *
 *   – pskCallback?(socket: TLSSocket, identity: string): DataView | NodeJS.TypedArray | null
 *   - pskIdentityHint?: string | undefined
 *
 */
export declare class TSServer {
    private static __server;
    private static readonly DEFAULT_CLOSE_TIMEOUT;
    readonly host: string;
    readonly port: number;
    readonly rootPath: string;
    readonly isHTTPs: boolean;
    private _serverOptions;
    private _httpServer;
    private _endPoints;
    private _sites;
    private _logger;
    private _logInfo;
    private _connections;
    private _terminating;
    private _forceCloseTimeout;
    static start(endPoints: Nullable<TSEndPointsDictionary>, opts?: Nullable<TSServerOptions>): Promise<TSServerStartStatus | Error>;
    static clearCaches(): Promise<void>;
    static stop(): Promise<Error | undefined>;
    private constructor();
    private _start;
    private _onConnection;
    private _onRequest;
    private _requestDidEnd;
    private _clearCaches;
    private _stop;
    private _internalStopServer;
}
export declare enum TSServerLogType {
    Log = "Info",
    Warning = "Warning",
    Error = "Error"
}
export type TSServerLogger = (server: TSServer, req: http.IncomingMessage | undefined, type: TSServerLogType, messages: string) => Promise<void>;
